@@ -30,7 +30,7 @@ use Config;
 use Digest::SHA;
 use Data::Dumper;
 use IPC::Open2;
-use File::Path qw(make_path);
+use File::Path qw(:DEFAULT make_path);
 use File::Basename;
 use Dpkg::Control::Hash;
 use Debian::Dgit::ExitStatus;
@@ -53,7 +53,7 @@ BEGIN {
                       stat_exists link_ltarget rename_link_xf
 		      hashfile
                       fail failmsg ensuredir ensurepath
-		      must_getcwd rmdir_r executable_on_path
+		      must_getcwd executable_on_path
                       waitstatusmsg failedcmd_waitstatus
 		      failedcmd_report_cmd failedcmd
                       runcmd runcmd_quieten
@@ -80,7 +80,6 @@ BEGIN {
                       initdebug enabledebug enabledebuglevel
                       printdebug debugcmd
                       $printdebug_when_debuglevel $debugcmd_when_debuglevel
-                      $atext_re $dot_atom_text_re $addr_spec_re $angle_addr_re
                       $debugprefix *debuglevel *DEBUG
                       shellquote printcmd messagequote
                       $negate_harmful_gitattrs
@@ -121,15 +120,6 @@ our $gdrlast_refprefix = 'debrebase-last';
 our $printdebug_when_debuglevel = 1;
 our $debugcmd_when_debuglevel = 1;
 
-# This is RFC 5322's 'atext'.
-our $atext_re         = qr([[:alnum:]!#$%&'*+\-/=?^_`{|}~])a;
-# This is RFC 5322's 'dot-atom-text' without comments and whitespace.
-our $dot_atom_text_re = qr($atext_re(?:\.|$atext_re)*)a;
-# This is RFC 5322's 'addr-spec' without obsolete syntax.
-our $addr_spec_re = qr($dot_atom_text_re\@$dot_atom_text_re);
-# This is RFC 5322's 'angle-addr' without obsolete syntax.
-our $angle_addr_re = qr(\<($addr_spec_re)\>)a;
-
 our (@git) = qw(git);
 
 # these three all go together, only valid after record_maindir
@@ -237,19 +227,6 @@ sub messagequote ($) {
 }
 
 sub shellquote {
-    # Quote an argument list for use as a fragment of shell text.
-    #
-    # Shell quoting doctrine in dgit.git:
-    #  * perl lists are always unquoted argument lists
-    #  * perl scalars are always individual arguments,
-    #    or if being passed to a shell, quoted shell text.
-    #
-    # So shellquote returns a scalar.
-    #
-    # When invoking ssh-like programs, that concatenate the arguments
-    # with spaces and then treat the result as a shell command, we never
-    # use the concatenation.  We pass the intended script as a single
-    # parameter (which is in accordance with the above doctrine).
     my @out;
     local $_;
     defined or confess __ 'internal error' foreach @_;
@@ -268,7 +245,7 @@ sub shellquote {
 sub printcmd {
     my $fh = shift @_;
     my $intro = shift @_;
-    print $fh $intro." ".(shellquote @_)."\n" or confess "$!";
+    print $fh $intro." ".(join '', shellquote @_)."\n" or confess "$!";
 }
 
 sub debugcmd {
@@ -574,9 +551,9 @@ sub hashfile ($) {
 }
 
 sub git_rev_parse ($;$) {
-    my ($ref, $cmd_map) = @_;
-    $cmd_map //= sub { @_; };
-    return cmdoutput $cmd_map->(qw(git rev-parse), "$ref~0");
+    my ($ref, $cmdprefix) = @_;
+    $cmdprefix //= [ ];
+    return cmdoutput @$cmdprefix, qw(git rev-parse), "$ref~0";
 }
 
 sub changedir_git_toplevel () {
@@ -808,49 +785,6 @@ sub changedir ($) {
     chdir $newdir or confess "chdir: $newdir: $!";
 }
 
-sub rmdir_r ($) {
-    my ($dir) = @_;
-    # Removes the whole subtree $dir (which need not exist), or dies.
-    #
-    # We used to use File::Path::remove_tree (via ::rmtree) but its
-    # error handling and chmod behaviour is very complex and confusing.
-    # For example:
-    #   - It chdirs, and then tries to chdir back.
-    #     With our $SIG{__WARN__} setting to die, this can, in combination
-    #     with eval, cause execution to continue in an unexpected cwd!
-    #   - Without the `safe` option it chmods things.  We never want that.
-    #   - *With* the `safe` option it appears to silently skip things
-    #     it would want to chdir.
-    #   - The error handling with the errors option is rather janky.
-
-    # Don't fork/exec rm if we don't have to.
-    return unless stat_exists $dir;
-
-    # We don't use runcmd because we want to capture errors from rmdir
-    # in $@ so that if we eval a rmdir_r, the right things happen.
-    # even cmdoutput_errok is too cooked.
-    my @cmd = (qw(rm -rf --), $dir);
-    debugcmd '+', @cmd;
-    my $child = open P, "-|" // confess $!;
-    if (!$child) {
-	open STDERR, ">& STDOUT" or die $!;
-	exec @cmd or die "exec $cmd[0]: $!";
-    }
-    $!=0; $?=0;
-    my $errs;
-    { local $/ = undef; $errs = <P>; }
-    confess "$!" if P->error;
-
-    return if close P && !stat_exists $dir;
-
-    chomp $errs;
-    $errs =~ s{\n}{; }g;
-    $errs ||= 'no error messages';
-
-    die f_ "failed to remove directory tree %s: rm -rf: %s; %s\n",
-      $dir, waitstatusmsg, $errs;
-}
-
 sub git_slurp_config_src ($) {
     my ($src) = @_;
     # returns $r such that $r->{KEY}[] = VALUE
@@ -930,13 +864,12 @@ sub parsecontrol {
 }
 
 sub parsechangelog {
-    # parsechangelog            @dpkg_changelog_args
-    # parsechangelog \&cmd_map, @dpkg_changelog_args
+    # parsechangelog [ \@cmdprefix ], @dpkg_changelog_args
     my $c = Dpkg::Control::Hash->new(name => 'parsed changelog');
     my $p = new IO::Handle;
-    my $cmd_map = sub { @_; };
-    $cmd_map = shift @_ if ref $_[0];
-    my @cmd = $cmd_map->(qw(dpkg-parsechangelog), @_);
+    my $prefix = [ ];
+    $prefix = shift @_ if ref $_[0];
+    my @cmd = (@$prefix, qw(dpkg-parsechangelog), @_);
     open $p, '-|', @cmd or confess "$!";
     $c->parse($p);
     $?=0; $!=0; close $p or failedcmd @cmd;
@@ -1161,7 +1094,7 @@ END
 #    playtree_setup
 #            # ^ call in some (perhaps trivial) subdir of playground
 #
-#    rmdir_r playground
+#    rmtree playground
 
 # ----- maindir -----
 
@@ -1209,7 +1142,7 @@ sub ensure_a_playground ($) {
 sub fresh_playground ($) {
     my ($spc) = @_;
     $spc = ensure_a_playground_parent $spc;
-    rmdir_r $spc;
+    rmtree $spc;
     mkdir $spc or fail f_
 	"failed to mkdir the playground %s: %s", $spc, $!;
     return $spc;
